
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const FileDescriptor_cxx_Version = 
"$Id: FileDescriptor.cxx,v 1.8 2000/08/11 00:32:59 bogawa Exp $";
#include "FileDescriptor.hxx"
#include "VLog.hxx"
#include "TransportException.hxx"
#include <unistd.h>
#include <fcntl.h>
#include <cerrno>

using namespace Vovida;

const file_descriptor_t	    FileDescriptor::INVALID = -1;

FileDescriptor::FileDescriptor(file_descriptor_t fd)
    :	fd_(fd)
{
}


FileDescriptor::~FileDescriptor()
throw ( TransportException )
{
    if ( fd_ != INVALID )
    {
    	close();
    }
}


file_descriptor_t   	
FileDescriptor::getFD() const
{
    return ( fd_ );
}

void  	    	
FileDescriptor::setBlocking()
throw ( TransportException )
{
    const string    fn("FileDescriptor::setBlocking");
    VLog    	    log(fn);
    
    int flags = fcntl(fd_, F_GETFL);
    
    if	(   flags < 0 
    	||  fcntl(fd_, F_SETFL, flags & ~O_NONBLOCK) <  0
    	)
    {
    	throw TransportException(fn + " on fcntl(): " + strerror(errno), 
	    	    	    	__FILE__, __LINE__, errno);
    }

    VDEBUG(log) << (fn + ": fd = ") << fd_ << VDEBUG_END(log);
}


void  	    	
FileDescriptor::setNonblocking()
throw ( TransportException )
{
    const string    fn("FileDescriptor::setNonblocking");
    VLog    	    log(fn);

    int flags = fcntl(fd_, F_GETFL);
    
    if  (   flags < 0 
    	||  fcntl(fd_, F_SETFL, flags | O_NONBLOCK) < 0
    	)
    {
    	throw TransportException(fn + " on fcntl(): " + strerror(errno), 
	    	    	    	__FILE__, __LINE__, errno);
    }

    VDEBUG(log) << (fn + ": fd = ") << fd_ << VDEBUG_END(log);
}


void  	    	
FileDescriptor::close()
throw ( TransportException )
{
    const string    fn("FileDescriptor::close");
    VLog    	    log(fn);

    if ( std::close(fd_) < 0 )
    {
    	throw TransportException(fn + " on std::close(): " + strerror(errno), 
	    	    	    	__FILE__, __LINE__, errno);
    }

    VDEBUG(log) << (fn + ": fd = ") << fd_ << VDEBUG_END(log);

    fd_ = INVALID;
}


bool	    	    
FileDescriptor::operator==(const FileDescriptor & rhs) const
{
    return ( fd_ == rhs.fd_ );
}


bool	    	    
FileDescriptor::operator!=(const FileDescriptor & rhs) const
{
    return ( fd_ != rhs.fd_ );
}


bool	    	    
FileDescriptor::operator<(const FileDescriptor & rhs) const
{
    return ( fd_ < rhs.fd_ );
}


bool	    	    
FileDescriptor::operator<=(const FileDescriptor & rhs) const
{
    return ( fd_ <= rhs.fd_ );
}


bool	    	    
FileDescriptor::operator>(const FileDescriptor & rhs) const
{
    return ( fd_ > rhs.fd_ );
}


bool	    	    
FileDescriptor::operator>=(const FileDescriptor & rhs) const
{
    return ( fd_ >= rhs.fd_ );
}


ostream &       
FileDescriptor::writeTo(ostream & out) const
{
    return ( out << "fd = " << fd_ );
}
