#if !defined(IPADDRESS_DOT_H)
#define IPADDRESS_DOT_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const IPAddress_hxx_Version = 
"$Id: IPAddress.hxx,v 1.16 2000/08/11 00:32:59 bogawa Exp $";

#include <netinet/in.h>

#include "TransportAddress.hxx"
#include <string>

namespace Vovida
{

class IPAddress : public TransportAddress
{
    public:
    	/// Constructors, by port, by dotted decimal and port, by octet string
	//  and port, by unsigned long and port, by sockaddr_in and by copy.
	//
    	    	    	    	IPAddress(u_int16_t 	    	port = 0);
			    	IPAddress(const char 	    *	ip,
			    	          u_int16_t 	    	port = 0);
			    	IPAddress(const u_int8_t    *   ip,
			    	          u_int16_t 	    	port = 0);
			    	IPAddress(u_int32_t    	    	ip,
			    	          u_int16_t 	    	port);
			    	IPAddress(const sockaddr_in &   inAddr);
			    	IPAddress(const IPAddress 	&);
			    
	virtual     	    	~IPAddress();
	
	/// Assignment operator
	///
	IPAddress & 	    	operator=(const IPAddress &);

    	/// Factory method. clone the current address.
	///
	TransportAddress    *	clone() const;
		
	/// Set entire address by sockaddr_in
	///
	void	    	    	setIPAddress(const sockaddr_in &);
	
	/// Set IP portion of address, by dotted decimal, by octet sting,
	/// by unsigned long.
	///
	void	    	    	setIP(const char *);
	void	    	    	setIP(const u_int8_t *);
	void	    	    	setIP(u_int32_t);
	
	/// Get IP porion of the address, by dotted decimal notation,
    	/// by octet sting (assuming dst points to 4-byte memory area),
	/// by unsigned long.
	///
	string	    	    	getIPAsString() const;
	void     	        getIPAsOctetString(u_int8_t* dst) const;
	u_int32_t               getIPAsULong() const;
    	string	    	    	getIPAndPortAsString() const;
		
	/// Set port portion of address.
	///
	void	    	    	setPort(u_int16_t);
    
    	/// Get the entire address, length.
	///
    	const sockaddr_in &	getIPAddress() const;
   	socklen_t  	    	getAddressLength() const;
	
	/// Get the port portion of the address.
	///
	u_int16_t   	    	getPort() const;

    	/// Update the address from the socket.
	//
	virtual void	    	updateAddress(const Socket &)
	    	    	    	    	throw ( TransportException );

    	/// Relational operators
	///
	bool	    	    	operator==(const IPAddress &) const;
	bool	    	    	operator!=(const IPAddress &) const;
	bool	    	    	operator<(const IPAddress &) const;
	bool	    	    	operator<=(const IPAddress &) const;
	bool	    	    	operator>(const IPAddress &) const;
	bool	    	    	operator>=(const IPAddress &) const;

    	/// Returns true if the addr portion is INADDR_ANY and the
	/// port is zero.
	///
    	bool	    	    	isAny() const;
	
    	/// Print the address to a stream.
	///	
	ostream &	    	writeTo(ostream &) const;
	
    private:

    	char *	    	    	inet_ntoa(const struct in_addr & in, char *) const;
	    
    	mutable sockaddr_in 	inAddr_;
};

} // namespace Vovida

#endif // !defined(IPADDRESS_DOT_H)
