#if !defined(PRIORITY_LOG_DOT_H)
#define PRIORITY_LOG_DOT_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const PriorityLog_hxx_Version = 
"$Id: PriorityLog.hxx,v 1.7 2000/08/11 00:33:00 bogawa Exp $";

#include "Writer.hxx"

#include <string>
#include <strstream>

namespace Vovida
{

class PriorityLog
{
    protected:
    	    	    	    	    PriorityLog(
				    	const char  	*   priorityName,
					int 	    	    priority,
					bool	    	    alwaysOn	= false,
					bool	    	    defaultOn 	= true);
	
	    	    	    	    PriorityLog(const PriorityLog &);
    public:
    					
	virtual     	    	    ~PriorityLog();

    	virtual PriorityLog *	    clone() const = 0;
	
		ostream &	    get(bool header = false);
	
	    	bool	    	    isOn() const;
		void	    	    on();
		void	    	    off();

    	virtual void	    	    preWrite();
	virtual void	    	    postWrite();

    	    	void	    	    end(const char  	*   file = 0, 
					int 	    	    line = 0);
	
	virtual void	    	    writeLog(
	    	    	    	    	const string	&   priorityName,
					int 	    	    priority,
					const char  	*   logMessage,
					const char  	*   file = 0, 
					int 	    	    line = 0) = 0;
    private:

    	    	ostrstream  	*   buffer_;
    	    	string	    	    priorityName_;
		int 	    	    priority_;
		bool	    	    alwaysOn_;
		bool	    	    on_;

    	    	// Suppress assignment
		//
    	    	PriorityLog &	    operator=(const PriorityLog &);
};


} // namespace Vovida

#endif // !defined(PRIORITY_LOG_DOT_H)
