#if !defined(PROTOCOL_DOT_H)
#define PROTOCOL_DOT_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const Protocol_hxx_Version = 
"$Id: Protocol.hxx,v 1.9 2000/08/11 00:33:00 bogawa Exp $";

#include "Writer.hxx"
#include "ProtocolException.hxx"
#include <string>

namespace Vovida
{

class FileDescriptor;
class ConnectionBrokenException;
class TransportException;

/// A Protocol is the object that gives meaning to the bytes
/// available from a FileDescriptor.
///
/// The Poll and Protocol classes together form a type of Subject/Observer
/// (see Design Patterns, Gamma, et al. page 293) pattern. The Poll object
/// is the subject whereas the Protocol is the Observer. The Protocol object
/// register's itself with the Poll. The protocol is interested in being 
/// notified when it's file descriptor is active. Thus the observer provides 
/// and an aspect to the server to reduce messaging activity. As well, since 
/// the observer provides an aspect, the push model is used.
///
class Protocol : public Writer
{
    public:
    	    	    	    	    	    Protocol(const char * name = 0);
    	virtual     	    	    	    ~Protocol();
	
	virtual const FileDescriptor &	    getFileDescriptor() const = 0;
	
	virtual void	    	    	    onIncomingAvailable();
	virtual void	    	    	    onOutgoingAvailable();
	virtual void	    	    	    onPriority();

	virtual void	    	    	    onDisconnect(
	    	    	    	    	    	ConnectionBrokenException * = 0
					    )
	    	    	    	    	    throw ( ProtocolException );

	virtual void	    	    	    onError(
	    	    	    	    	    	TransportException * = 0
	    	    	    	    	    )
	    	    	    	    	    throw ( ProtocolException );

    	virtual void	    	    	    handleUnknownException();
					    
		void	    	    	    awaitIncoming(bool);
		bool	    	    	    awaitIncoming() const;

    	    	void	    	    	    awaitOutgoing(bool);
    	    	bool	    	    	    awaitOutgoing() const;

		void	    	    	    awaitPriority(bool);
		bool	    	    	    awaitPriority() const;

	virtual ostream &           	    writeTo(ostream &) const;

    	    	bool	    	    	    operator==(const Protocol &);
		
    private:
		bool	    	    	    incoming_;
    	    	bool	    	    	    outgoing_;
		bool	    	    	    priority_;
		string      	    	    name_;
		
    	// Suppress copying
	//
	    	    	    	    	    Protocol(const Protocol &);
	Protocol &  	    	    	    operator=(const Protocol &);
};

} // namespace Vovida

#endif // !defined(PROTOCOL_DOT_H)
