#if !defined(THREAD_DOT_H)
#define THREAD_DOT_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const Thread_hxx_Version = 
"$Id: Thread.hxx,v 1.9 2000/08/11 00:33:01 bogawa Exp $";

#include "Writer.hxx"
#include "TransportCommon.hxx"
#include "VThread.h"
#include "VMutex.h"
#include <hash_map>

namespace Vovida
{

class Runnable;
class SignalHandler;

class Thread : public Writer
{
    public:
    
    	static	void	    	    init();
	static	void	    	    uninit();

    	static	Thread *    	    self();
    	static	vthread_t   	    selfId();
	
    	/// Start a new thread of control. 
	///
    	Thread(Runnable &, const char * name = 0);
				    
	/// Delete Thread object. This does not stop the thread
	/// of control.
	///
    	~Thread();

    	/// Join the thread. Must be called from outside the thread of
	/// execution. Returns the result of the Runnable's run() method.
	///
	ReturnCode  	    	    join();
	
	/// Exit the thread. Must be called from within the thread
	/// of execution.
	///
	void	    	    	    exit();
	
	/// Return the runnable associated with this thread.
	///
	Runnable &  	    	    runnable();
	
	/// Return the signalHandler associated with this thread.
	///
	SignalHandler &     	    signalHandler();
	
    	/// Write thread information to stream. See Writer.
	///
	ostream &           	    writeTo(ostream &) const;
	
	
    private:

    	/// The function that is run when the thread is spawned.
	///
    	static	void *	    	    threadMain(void * data);

    	/// Default constructor for main thread.
	///
    	Thread();
		
	typedef hash_map<vthread_t, Thread *>	    ThreadMap;
	
    	static	ThreadMap   			*   threadMap_;
    	static	VMutex	    	    	    	    threadMapMutex_;
	static	Thread      	    	     	*   mainThread_;
	
	    	VThread     	    	    	*   thread_;
		Runnable    	    	    	&   runnable_;
		SignalHandler	    	    	*   signalHandler_;
		string	    	    	    	    name_;
};

} // namespace Vovida

#endif // !defined(THREAD_DOT_H)
