#if !defined(VLOG_DOT_H)
#define VLOG_DOT_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const VLog_hxx_Version = 
"$Id: VLog.hxx,v 1.8 2000/08/11 00:33:02 bogawa Exp $";

#include "Log.hxx"
#include "CpPriorityLog.hxx"

#include <string>

#define     LOG_TRACE	    	    100
#define     LOG_VERBOSE	    	    101

namespace Vovida
{

class VLog : public Log
{
    public:
    	    	    	    	    VLog();
    	    	    	    	    VLog(const string & func_name);
	virtual     	    	    ~VLog();
	
	static	void  	    	    init(
	    	    	    	    	int 	    	priorityLevel,
					const char *	file = 0);
					
	static	void 	    	    uninit();
	
    	static	void	    	    on(int);
	static	void	    	    off(int);

		void		    logOn(int);
    	    	void	    	    logOff(int);
		
		PriorityLog &	    logEmergency();
		PriorityLog &	    logAlert();
		PriorityLog &	    logCritical();
		PriorityLog &	    logError();
		PriorityLog &	    logWarning();
		PriorityLog &	    logNotice();
		PriorityLog &	    logInfo();
		PriorityLog &	    logDebug();
		PriorityLog &	    logTrace();
		PriorityLog &	    logVerbose();
		

    private:

    	static	Sptr<PriorityLog>   null_;
    	static	bool	    	    traceOn_;
    	static	bool	    	    verboseOn_;

    	    	Sptr<PriorityLog>   emergency_;
    	    	Sptr<PriorityLog>   alert_;
    	    	Sptr<PriorityLog>   critical_;
		Sptr<PriorityLog>   error_;
    	    	Sptr<PriorityLog>   warning_;
    	    	Sptr<PriorityLog>   notice_;
    	    	Sptr<PriorityLog>   info_;
    	    	Sptr<PriorityLog>   debug_;
		bool	    	    trace_;
		bool	    	    verbose_;
		
		string	    	    funcName_;
};  	

#define     VEMERG(x)	    if ( !((x).logEmergency().isOn()) );\
    	    	    	    else (x).logEmergency().get(true)
#define     VALERT(x)	    if ( !((x).logAlert().isOn()) );\
    	    	    	    else (x).logAlert().get(true)
#define     VCRIT(x)	    if ( !((x).logCritical().isOn()) );\
    	    	    	    else (x).logCritical().get(true)
#define     VERR(x)	    if ( !((x).logError().isOn()) );\
    	    	    	    else (x).logError().get(true)
#define     VWARN(x)	    if ( !((x).logWarning().isOn()) );\
    	    	    	    else (x).logWarning().get(true)
#define     VNOTICE(x)	    if ( !((x).logNotice().isOn()) );\
    	    	    	    else (x).logNotice().get(true)
#define     VINFO(x)	    if ( !((x).logInfo().isOn()) );\
    	    	    	    else (x).logInfo().get(true)
#define     VDEBUG(x)	    if ( !((x).logDebug().isOn()) );\
    	    	    	    else (x).logDebug().get(true)
#define     VTRACE(x)	    if ( !((x).logTrace().isOn()) );\
    	    	    	    else (x).logTrace().get(true)
#define     VVERBOSE(x)	    if ( !((x).logVerbose().isOn()) );\
    	    	    	    else (x).logVerbose().get(true)


#define     VEMERG_CONT(x)  if ( !((x).logEmergency().isOn()) );\
    	    	    	    else (x).logEmergency().get()
#define     VALERT_CONT(x)  if ( !((x).logAlert().isOn()) );\
    	    	    	    else (x).logAlert().get()
#define     VCRIT_CONT(x)   if ( !((x).logCritical().isOn()) );\
    	    	    	    else (x).logCritical().get()
#define     VERR_CONT(x)    if ( !((x).logError().isOn()) );\
    	    	    	    else (x).logError().get()
#define     VWARN_CONT(x)   if ( !((x).logWarning().isOn()) );\
    	    	    	    else (x).logWarning().get()
#define     VNOTICE_CONT(x) if ( !((x).logNotice().isOn()) );\
    	    	    	    else (x).logNotice().get()
#define     VINFO_CONT(x)   if ( !((x).logInfo().isOn()) );\
    	    	    	    else (x).logInfo().get()
#define     VDEBUG_CONT(x)  if ( !((x).logDebug().isOn()) );\
    	    	    	    else (x).logDebug().get()
#define     VTRACE_CONT(x)  if ( !((x).logTrace().isOn()) );\
    	    	    	    else (x).logTrace().get()
#define     VVERBOSE_CONT(x) if ( !((x).logVerbose().isOn()) );\
    	    	    	     else (x).logVerbose().get()

#define     VEMERG_END(x)   ends; (x).logEmergency().end(__FILE__,__LINE__)
#define     VALERT_END(x)   ends; (x).logAlert().end(__FILE__,__LINE__)
#define     VCRIT_END(x)    ends; (x).logCritical().end(__FILE__,__LINE__)
#define     VERR_END(x)     ends; (x).logError().end(__FILE__,__LINE__)
#define     VWARN_END(x)    ends; (x).logWarning().end(__FILE__,__LINE__)
#define     VNOTICE_END(x)  ends; (x).logNotice().end(__FILE__,__LINE__)
#define     VINFO_END(x)    ends; (x).logInfo().end(__FILE__,__LINE__)
#define     VDEBUG_END(x)   ends; (x).logDebug().end(__FILE__,__LINE__)
#define     VTRACE_END(x)   ends; (x).logTrace().end(__FILE__,__LINE__)
#define     VVERBOSE_END(x) ends; (x).logVerbose().end(__FILE__,__LINE__)

} // namespace Vovida

#endif // !defined(VLOG_DOT_H)
